<?php

namespace Drupal\graphql_flag\Plugin\GraphQL\Fields;

use Drupal\Core\Entity\FieldableEntityInterface;
use Drupal\graphql\GraphQL\Execution\ResolveContext;
use Drupal\graphql\Plugin\GraphQL\Fields\FieldPluginBase;
use GraphQL\Type\Definition\ResolveInfo;

/**
 * Amount of flaggings per flag for an entity.
 *
 * Can be used for "like" use cases.
 *
 * @GraphQLField(
 *   id = "entity_flagging_count",
 *   name = "entityFlaggingCount",
 *   description = @Translation("Amount of flaggings per flag for an entity."),
 *   secure = true,
 *   type = "EntityFlaggingCountList",
 *   parents = {
 *     "Entity",
 *   },
 *   arguments = {
 *     "flag_id" = "[String]"
 *   },
 * )
 */
class EntityFlaggingCount extends FieldPluginBase {

  /**
   * {@inheritdoc}
   */
  public function resolveValues($value, array $args, ResolveContext $context, ResolveInfo $info) {
    // @todo check if entity type / bundle has flag.
    // @todo handle optional argument to filter by flag id's.
    // @todo dependency injection.
    if ($value instanceof FieldableEntityInterface) {
      /** @var \Drupal\flag\FlagCountManagerInterface $flagCountService */
      $flagCountService = \Drupal::service('flag.count');
      $counts = $flagCountService->getEntityFlagCounts($value);

      // The flag count service does not return the flag if it is empty.
      // Make sure to have empty values as well by iterating over the flags.
      /** @var \Drupal\flag\FlagServiceInterface $flagService */
      $flagService = \Drupal::service('flag');
      $flags = $flagService->getAllFlags($value->getEntityTypeId(), $value->bundle());
      $flagIds = [];
      /** @var \Drupal\flag\FlagInterface $flag */
      foreach ($flags as $flag) {
        $flagIds[] = $flag->id();
      }

      $flaggingCounts = [];
      $flaggingCountsFlagIds = [];
      // Set initial values from the flag.count service.
      foreach ($counts as $flagId => $count) {
        $flaggingCounts[] = [
          'flagId' => $flagId,
          'amount' => (int) $count,
        ];
        $flaggingCountsFlagIds[] = $flagId;
      }
      // Initialize with flags that have no values.
      $initializeFlagIds = array_diff($flagIds, $flaggingCountsFlagIds);
      foreach ($initializeFlagIds as $flagId) {
        $flaggingCounts[] = [
          'flagId' => $flagId,
          'amount' => 0,
        ];
      }

      yield [
        'entity_flagging_counts' => $flaggingCounts,
      ];
    }
  }

}
